#!/usr/bin/perl
#
# Send-Uucp.pl - create news batches from the outgoing files
# This is a rewritten version from the send-uucp shell script that is
# shipped with INN. Four times faster :-)
# It's a little more verbose when creating the batches.
#
# Author: Edvard Tuinder <ed@elm.net>
#
# Copyright (C) 1994 Edvard Tuinder - The OpenWorld Foundation
# Copyright (C) 1995-1997 Miquel van Smoorenburg - Cistron Internet Services
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version
# 2 of the License, or (at your option) any later version.
#

require '/usr/lib/news/lib/innshellvars.pl';

$CF_FILE = '/etc/news/send-uucp.cf';
$BATCH_LOG = $inn'most_logs . '/news.admin';
$MAXSIZE = 150000;

$Vers = 1.16;

sub logit {
  print LOG "$PROGNAME: \[$$\] @_\n";
  #print "$PROGNAME: \[$$\] $_[0]\n";
}

sub do_end {
  unlink $LOCK;
  $ct = localtime(time);
  printf LOG "$PROGNAME: \[$$\] end $ct\n";
  exit 0;
}

sub read_cf {
  local (@TIME, $I, $hour);

  $hour = (localtime(time))[2];

  unless (open (CF, $CF_FILE)) {
    &logit ("can't open `$CF_FILE'");
    return 1;
  }
  while (<CF>) {
    chop;
    s/\s*#.*$//;
    unless (/^$/) {
      ($Site, $Compress, $Size, $Time) = split (/\s+/);

      $Compress = $inn'compress if ($Compress eq "");
      $Size = $MAXSIZE if ($Size eq "");

      if ($Want_Site eq $Site) {
        push (@SITELIST, $Site);
        push (@COMPLIST, $Compress);
        push (@SIZELIST, $Size);
	close CF;
	return 0;
      }
      if ($Want_Site eq "") {
        if ($Time ne "") {
          @TIME = split (',', $Time);
          $I = 0;
          while ($I <= $#TIME) {
            if ($TIME[$I] == $hour) {
              push (@SITELIST, $Site);
              push (@COMPLIST, $Compress);
              push (@SIZELIST, $Size);
            }
            $I++;
          }
        } else {
          push (@SITELIST, $Site);
          push (@COMPLIST, $Compress);
          push (@SIZELIST, $Size);
        }
      }
    }
  }
  close CF;
  return 0;
}

# Main section
$PROGNAME='send-uucp';
$LOCK = $inn'locks."/LOCK.$PROGNAME";
$LOG = $inn'most_logs."/$PROGNAME.log";

$ENV{'PATH'}='/usr/lib/news/bin:/bin:/usr/bin:/sbin:/usr/sbin';

$MAXJOBS = 200;
$UUXFLAGS = '- -z -r -gd';
unless (open (LOG, ">> $LOG")) {
  print "$PROGNAME: \[$$\] Couldn't open log file\n";
  exit 0;
}

$ct = localtime(time);
printf LOG "$PROGNAME: \[$$\] begin $ct\n";
unless (chdir ($inn'batch)) {
  &logit ("Can't access $inn'batch: $!");
  exit 0;
}

if (-e $LOCK) {
   unless (open(LOCK, $LOCK)) {
     &logit ("can't open $LOCK");
     close LOG;
     exit 0;
   }
   $_ = <LOCK>;
   s/[\r\n]+$//;
   &logit ("locked by $_");
   close LOCK;
   close LOG;
   exit 0;
}

#
# FIXME: this should be atomic.
#
unless (open(LOCK, "> $LOCK")) {
  &logit ("can't create lockfile $LOCK");
  close LOG;
  exit 0;
}
print LOCK "$$";
close LOCK;

if ($#ARGV != -1) {
  @LIST = @ARGV;
  $I = 0;
  while ($I <= $#LIST) {
    $Want_Site = $LIST[$I++];
    $ret = &read_cf;
    if ($ret == 1 || $#SITELIST == -1) {
      print "Site `$Want_Site' not found in config file. Ignored.\n";
    }
  }
} else {
  $ret = &read_cf;
  if ($#SITELIST == -1 || $ret == 1) {
    &logit ("nothing to do.");
    &do_end;
  }
}

# Perform the real work
&run_site while ($Site = pop(@SITELIST));
&do_end;

#end of main section
sub run_site {
  &logit ("checking site $Site");
  rename ($Site, "$Site.work");
  system ("$inn'newsbin/ctlinnd -s -t30 flush $Site");
  if (-z "$Site.work") {
    &logit ("no articles for $Site");
    $Size = pop @SIZELIST;	# still need to remove them @%^&# entries !
    $Compress = pop @COMPLIST;
    unlink ("$Site.work");
  } else {
    $Size = pop @SIZELIST;
    $Compress = pop @COMPLIST;
    if ($Compress cmp "none") {
      system ("batcher -v -b $Size -p\"(echo '#! cunbatch' ; exec $Compress ) | uux $UUXFLAGS %s!rnews\" $Site $Site.work >> $BATCH_LOG");
    } else {
      system ("batcher -v -b $Size -p\"uux $UUXFLAGS %s!rnews\" $Site $Site.work >> $BATCH_LOG");
    }
    unlink ("$Site.work");
    &logit ("batched for $Site");
  }
}
